;;; -*-  Mode: LISP -*- Syntax: AutoLISP (C) Benjamin Olasov 1988
;;;      low level entity access and modification

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; File: CHG.LSP Copyright (C) Benjamin Olasov 1988 All Rights Reserved    ;;;
;;; Research/ commercial inquiries:                                         ;;;
;;;       Benjamin Olasov  310 Riverside Drive    New York, NY  10025       ;;;
;;;                        PH (212) 678-5473                                ;;;
;;;                        Network Mailstop: G.OLASOV@CS.COLUMBIA.EDU       ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  C:CHG displays and modifies the properties of enitites while in the    ;;
;;  drawing mode.  Its modifications can generally be undone by running it ;;
;;  again and restoring the property's previous value.  There are some     ;;
;;  interesting effects of using CHG:  in a drawing containing two valid   ;;
;;  blocks A and B, an individual iteration of block A can be transformed  ;;
;;  to an iteration of block B by giving B as its new name.  All of its    ;;
;;  previous insertion parameters will remain the same, but its identity   ;;
;;  will be changed to block B.  If the name of the layer in which the     ;;
;;  entity resides is changed to the name of an existing layer, the entity ;;
;;  will change its residence to that layer.  However, if the new layer    ;;
;;  name is the name of a non-existing layer, a layer with that name will  ;;
;;  be created, and the entity will be transferred to that layer.          ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(gc)
(textscr)
(prompt "\nFile: CHG.LSP (C) Benjamin Olasov New York, NY 1987 All rights reserved.")
(prompt "\nThis function allows low level entity access and modification.")

(defun verbose-descriptor (key)
        (cond ((= key 0) "ENTITY TYPE: ")
              ((= key 1) "TEXT VALUE: ")
              ((and (= key 2)
                    (= (cdr (assoc 0 entity)) "ATTDEF")) "ATTRIBUTE TAG: ")
              ((and (= key 2)
                    (= (cdr (assoc 0 entity)) "INSERT")) "BLOCK NAME: ")
              ((= key 2) "NAME: ")
              ((and (>= key 3)
                    (<= key 5)) "OTHER NAME VALUES: ")
              ((= key 6) "LINETYPE NAME <RO>: ")
              ((= key 7) "TEXT STYLE NAME <RO>: ")
              ((= key 8) "LAYER: ")
              ((= key 9) "VARIABLE NAME IDENTIFIER: ")
              ((and (= key 10)
                    (= (cdr (assoc 0 entity)) "INSERT")) "INSERTION BASE: ")
              ((= key 10) "ORIGIN POINT: ")
              ((and (>= key 11)
                    (<= key 18)) "OTHER POINT COORDINATES: ")
              ((= key 20) "PRIMARY Y COORDINATE: ")
              ((and (>= key 21) (<= key 28)) "OTHER Y COORDINATES: ")
              ((and (>= key 31) (<= key 36)) "OTHER Z COORDINATES: ")
              ((= key 38) "ELEVATION: ")
              ((= key 39) "THICKNESS: ")
              ((and (>= key 40)
                    (<= key 48)
                    (or (= (CDR (ASSOC 0 ENTITY)) "CIRCLE")
                        (= (CDR (ASSOC 0 ENTITY)) "ARC"))) "RADIUS: ")
              ((and (>= key 40)
                    (<= key 48)
                    (or (= (cdr (assoc 0 entity)) "TEXT")
                        (= (cdr (assoc 0 entity)) "ATTDEF"))) "TEXT HEIGHT: ")
              ((and (= key 41)
                    (= (cdr (assoc 0 entity)) "INSERT")) "X SCALE FACTOR: ")
              ((and (= key 42)
                    (= (cdr (assoc 0 entity)) "INSERT")) "Y SCALE FACTOR: ")
              ((and (= key 43)
                    (= (cdr (assoc 0 entity)) "INSERT")) "Z SCALE FACTOR: ")
              ((and (>= key 40)
                    (<= key 48)) "FLOATING POINT VALUES: ")
              ((= key 49) "REPEATED VALUE: ")
              ((and (>= key 50)
                    (<= key 58)) "ANGLE: ")
              ((= key 62) "COLOR NUMBER <RO>: ")
              ((= key 66) "ENTITIES FOLLOW <RO>: ")
              ((= key 71) "MIRROR DIRECTION: ")
              ((and (>= key 70) (<= key 78)) "INTEGER VALUES: ")))

(defun format-input (key)
        (cond ((= (type (cdr (assoc key entity))) (read "STR"))
               (graphscr)
               (princ (strcat "\nCurrent string: " (cdr (assoc key entity))))
               (if (or (= (cdr (assoc 0 entity)) "INSERT")
                       (= key 8)) 
                       (strcase (getstring (strcat "\nNew "
                                                   (cdr (assoc 0 entity))
                                                   " name: ")))
                       (getstring t "\nNew string: ")))
              ((= (type (cdr (assoc key entity))) (read "REAL"))
               (if (and (>= key 50) (<= key 58))
                   (progn
                        (setvar "coords" 2)
                        (graphscr)
                        (princ "\nCurrent angle: ")
                        (princ (cdr (assoc key entity)))
                        (getangle (cdr (assoc 10 entity)) "\nNew angle: "))
                   (progn
                        (graphscr)
                        (princ "\nCurrent real value: ")
                        (princ (cdr (assoc key entity)))
                        (getreal "\nNew real value: "))))
              ((= (type (cdr (assoc key entity))) (read "INT"))
               (princ "\nCurrent integer value: ")
               (princ (cdr (assoc key entity)))
               (getint "\nNew integer value: "))
              ((= (type (cdr (assoc key entity))) (read "LIST"))
               (graphscr)
               (setvar "coords" 2)
               (princ "\nCurrent point value: ")
               (princ (cdr (assoc key entity)))
               (getpoint (cdr (assoc key entity)) "\nNew point: "))))

(defun C:CHG ()
       (setq entity (cdr (setq ent (entget (setq ename (car (entsel))))))
             header (strcat (cdr (assoc 0 entity)) " PROPERTY TABLE")
             counter 0)
       (repeat 24 (terpri))
       (textscr)
       (repeat (- 38 (/ (strlen header) 2)) (princ "\260"))
       (princ (strcat " " header " "))
       (repeat (- 38 (/ (strlen header) 2)) (princ "\260"))
       (repeat (fix (/ (- 24 (length (cdr entity))) 2.0)) (terpri))
       (setq *entity* (cdr entity)
             **entity** (list (cons counter (cdr (assoc 0 entity)))))
       (while (car *entity*)
              (setq *key* (caar *entity*))
              (if (verbose-descriptor *key*)
                  (progn
                     (setq counter (1+ counter)
                           **entity** (cons (cons counter *key*) **entity**))
                     (princ (strcat (rtos (float counter) 2 0) "] "
                                    (verbose-descriptor *key*)))
                     (princ (cdr (assoc *key* *entity*)))
                     (terpri)))
              (setq *entity* (cdr *entity*)))
       (repeat (fix (/ (- 24 (length (cdr entity))) 2.0)) (terpri))
       (princ "Number of property to change: ")
       (setq *tag* (cdr (assoc (getint) **entity**))
             *new* (format-input *tag*)
             ent (subst (cons *tag* *new*) (assoc *tag* entity) ent))
       (entmod ent))

;;                           ** NEWS BULLETIN **
;;    
;;    New CAD/AutoCAD section on New York BBS Service Bulletin Board
;;    
;;    Discussions, news and software relevant to CAD, especially AutoCAD,
;;    are supported.  The primary aim of this section is to be a resource 
;;    for AutoLISP (and LISP) applications developers generally, with some
;;    emphasis on applying knowledge representation techniques in AutoCAD's
;;    LISP environment.  However, other forms of innovation, such as new 
;;    pattern files, text fonts- clever and useful applications generally- 
;;    are strongly encouraged.  Also, we very much need drawing libraries 
;;    such as steel sections, etc.  Many new, experimental and/or 
;;    research-oriented LISP applications will be made available on a 
;;    regular basis. 
;;
;;    The board is in New York City. The dial-up number is (212) 980-0770.
;;
;;    We need callers participation in developing this CAD section.
;;    At this stage, our library of CAD applications is still far too light.
;;    We need more uploads from callers, especially LISP routines.
;;    
;;    Direct technical questions/ comments to one of the following addresses:
;;    
;;    Arpanet Network Mailstop: G.OLASOV@CS.COLUMBIA.EDU
;;    MCI-MAIL ID 344-4003
;;    CompuServe ID 71450,3313

